@@ -6,9 +6,10 @@ use crate::{
 };
 use anyhow::Result;
 use crossterm::event::{self, EventStream, KeyCode, KeyModifiers};
+use rand::Rng;
 use rspotify::model::offset;
 use std::sync::mpsc;
-use tokio::stream::StreamExt;
+use tokio_stream::StreamExt;
 
 #[derive(Debug)]
 pub enum ContextURI {
@@ -19,24 +20,31 @@ pub enum ContextURI {
 }
 
 #[derive(Debug)]
-/// An event to communicate with the client
-pub enum Event {
-    GetDevices,
-    GetUserPlaylists,
-    GetUserSavedAlbums,
-    GetUserFollowedArtists,
-    GetCurrentPlayback,
-    RefreshToken,
+/// An event that modifies the player's playback
+pub enum PlayerEvent {
     NextTrack,
     PreviousTrack,
     ResumePause,
     SeekTrack(u32),
     Repeat,
     Shuffle,
-    GetContext(ContextURI),
+    Volume(u8),
     PlayTrack(Option<String>, Option<Vec<String>>, Option<offset::Offset>),
-    PlayContext(String),
-    TransferPlayback(String),
+}
+
+#[derive(Debug)]
+/// An event to communicate with the client
+/// TODO: renaming this enum (e.g to `ClientRequest`)
+pub enum Event {
+    RefreshToken,
+    GetDevices,
+    GetUserPlaylists,
+    GetUserSavedAlbums,
+    GetUserFollowedArtists,
+    GetContext(ContextURI),
+    GetCurrentPlayback,
+    TransferPlayback(String, bool),
+    Player(PlayerEvent),
 }
 
 impl From<event::KeyEvent> for Key {
@@ -52,8 +60,8 @@ impl From<event::KeyEvent> for Key {
 }
 
 #[tokio::main]
-/// starts the application's event stream that pools and handles events from the terminal
-pub async fn start_event_stream(send: mpsc::Sender<Event>, state: SharedState) {
+/// starts a terminal event handler
+pub async fn start_event_handler(send: mpsc::Sender<Event>, state: SharedState) {
     let mut event_stream = EventStream::new();
 
     while let Some(event) = event_stream.next().await {
@@ -221,8 +229,12 @@ fn handle_terminal_event(
                         command,
                         player.devices.len(),
                         |_: &mut UIStateGuard, _: usize| {},
-                        |_: &mut UIStateGuard, id: usize| -> Result<()> {
-                            send.send(Event::TransferPlayback(player.devices[id].id.clone()))?;
+                        |ui: &mut UIStateGuard, id: usize| -> Result<()> {
+                            send.send(Event::TransferPlayback(
+                                player.devices[id].id.clone(),
+                                true,
+                            ))?;
+                            ui.popup = PopupState::None;
                             Ok(())
                         },
                         |ui: &mut UIStateGuard| {
@@ -264,7 +276,7 @@ fn handle_mouse_event(
             if let Some(track) = track {
                 let position_ms =
                     track.duration_ms * (event.column as u32) / (ui.progress_bar_rect.width as u32);
-                send.send(Event::SeekTrack(position_ms))?;
+                send.send(Event::Player(PlayerEvent::SeekTrack(position_ms)))?;
             }
         }
     }
@@ -284,8 +296,27 @@ fn handle_command_for_none_popup(
             Ok(true)
         }
         Command::PlayContext => {
-            let uri = state.player.read().unwrap().context_uri.clone();
-            send.send(Event::PlayContext(uri))?;
+            let player = state.player.read().unwrap();
+            let context = player.get_context();
+
+            // randomly play a track from the current context
+            if let Some(context) = context {
+                if let Some(tracks) = context.get_tracks() {
+                    let offset = match context {
+                        // Spotify does not allow to manually specify `offset` for artist context
+                        Context::Artist(_, _, _, _) => None,
+                        _ => {
+                            let id = rand::thread_rng().gen_range(0..tracks.len());
+                            offset::for_uri(tracks[id].uri.clone())
+                        }
+                    };
+                    send.send(Event::Player(PlayerEvent::PlayTrack(
+                        Some(player.context_uri.clone()),
+                        None,
+                        offset,
+                    )))?;
+                }
+            }
             Ok(true)
         }
         Command::FocusNextWindow => {
@@ -490,23 +521,37 @@ fn handle_command(
             Ok(true)
         }
         Command::NextTrack => {
-            send.send(Event::NextTrack)?;
+            send.send(Event::Player(PlayerEvent::NextTrack))?;
             Ok(true)
         }
         Command::PreviousTrack => {
-            send.send(Event::PreviousTrack)?;
+            send.send(Event::Player(PlayerEvent::PreviousTrack))?;
             Ok(true)
         }
         Command::ResumePause => {
-            send.send(Event::ResumePause)?;
+            send.send(Event::Player(PlayerEvent::ResumePause))?;
             Ok(true)
         }
         Command::Repeat => {
-            send.send(Event::Repeat)?;
+            send.send(Event::Player(PlayerEvent::Repeat))?;
             Ok(true)
         }
         Command::Shuffle => {
-            send.send(Event::Shuffle)?;
+            send.send(Event::Player(PlayerEvent::Shuffle))?;
+            Ok(true)
+        }
+        Command::VolumeUp => {
+            if let Some(ref playback) = state.player.read().unwrap().playback {
+                let volume = std::cmp::min(playback.device.volume_percent + 5, 100_u32);
+                send.send(Event::Player(PlayerEvent::Volume(volume as u8)))?;
+            }
+            Ok(true)
+        }
+        Command::VolumeDown => {
+            if let Some(ref playback) = state.player.read().unwrap().playback {
+                let volume = std::cmp::max(playback.device.volume_percent as i32 - 5, 0_i32);
+                send.send(Event::Player(PlayerEvent::Volume(volume as u8)))?;
+            }
             Ok(true)
         }
         Command::OpenCommandHelp => {
@@ -604,17 +649,32 @@ fn handle_command_for_focused_context_window(
                     ArtistFocusState::RelatedArtists => {
                         handle_command_for_artist_list(command, send, ui, artists)
                     }
-                    ArtistFocusState::TopTracks => {
-                        handle_command_for_track_table(command, send, ui, state, tracks)
-                    }
+                    ArtistFocusState::TopTracks => handle_command_for_track_table(
+                        command,
+                        send,
+                        ui,
+                        None,
+                        Some(tracks.iter().map(|t| t.uri.clone()).collect::<Vec<_>>()),
+                        tracks,
+                    ),
                 }
             }
-            Context::Album(_, ref tracks) => {
-                handle_command_for_track_table(command, send, ui, state, tracks)
-            }
-            Context::Playlist(_, ref tracks) => {
-                handle_command_for_track_table(command, send, ui, state, tracks)
-            }
+            Context::Album(ref album, ref tracks) => handle_command_for_track_table(
+                command,
+                send,
+                ui,
+                Some(album.uri.clone()),
+                None,
+                tracks,
+            ),
+            Context::Playlist(ref playlist, ref tracks) => handle_command_for_track_table(
+                command,
+                send,
+                ui,
+                Some(playlist.uri.clone()),
+                None,
+                tracks,
+            ),
             Context::Unknown(_) => Ok(false),
         },
         None => Ok(false),
@@ -699,7 +759,8 @@ fn handle_command_for_track_table(
     command: Command,
     send: &mpsc::Sender<Event>,
     ui: &mut UIStateGuard,
-    state: &SharedState,
+    context_uri: Option<String>,
+    track_uris: Option<Vec<String>>,
     tracks: &[Track],
 ) -> Result<bool> {
     let tracks = ui.get_search_filtered_items(tracks);
@@ -723,31 +784,20 @@ fn handle_command_for_track_table(
         }
         Command::ChooseSelected => {
             if let Some(id) = ui.window.selected() {
-                match state.player.read().unwrap().get_context().unwrap() {
-                    Context::Artist(_, _, _, _) => {
-                        // cannot use artist context uri with a track uri
-                        let tracks = tracks.iter().map(|t| t.uri.clone()).collect::<Vec<_>>();
-                        send.send(Event::PlayTrack(
-                            None,
-                            Some(tracks),
-                            offset::for_position(id as u32),
-                        ))?;
-                    }
-                    Context::Playlist(ref playlist, _) => {
-                        send.send(Event::PlayTrack(
-                            Some(playlist.uri.clone()),
-                            None,
-                            offset::for_uri(tracks[id].uri.clone()),
-                        ))?;
-                    }
-                    Context::Album(ref album, _) => {
-                        send.send(Event::PlayTrack(
-                            Some(album.uri.clone()),
-                            None,
-                            offset::for_uri(tracks[id].uri.clone()),
-                        ))?;
-                    }
-                    Context::Unknown(_) => {}
+                if track_uris.is_some() {
+                    // play a track from a list of tracks, use ID offset for finding the track
+                    send.send(Event::Player(PlayerEvent::PlayTrack(
+                        None,
+                        track_uris,
+                        offset::for_position(id as u32),
+                    )))?;
+                } else if context_uri.is_some() {
+                    // play a track from a context, use URI offset for finding the track
+                    send.send(Event::Player(PlayerEvent::PlayTrack(
+                        context_uri,
+                        None,
+                        offset::for_uri(tracks[id].uri.clone()),
+                    )))?;
                 }
             }
             Ok(true)
