@@ -1,17 +1,21 @@
+use crate::token::Token;
 use rspotify::model::*;
 
 /// Player state
 #[derive(Debug)]
 pub struct PlayerState {
     pub devices: Vec<device::Device>,
-    pub auth_token_expires_at: std::time::SystemTime,
+    pub token: Token,
+
     pub user_playlists: Vec<playlist::SimplifiedPlaylist>,
     pub user_followed_artists: Vec<Artist>,
     pub user_saved_albums: Vec<Album>,
+
     pub context_uri: String,
     pub context_cache: lru::LruCache<String, Context>,
+
     pub playback: Option<context::CurrentlyPlaybackContext>,
-    pub playback_last_updated: Option<std::time::SystemTime>,
+    pub playback_last_updated: Option<std::time::Instant>,
 }
 
 /// Playing context (album, playlist, etc) of the current track
@@ -77,25 +81,17 @@ impl PlayerState {
     pub fn get_playback_progress(&self) -> Option<u32> {
         match self.playback {
             None => None,
-            Some(ref playback) => match playback.item {
-                Some(rspotify::model::PlayingItem::Track(ref track)) => {
-                    let progress_ms = (playback.progress_ms.unwrap() as u128)
-                        + if playback.is_playing {
-                            std::time::SystemTime::now()
-                                .duration_since(self.playback_last_updated.unwrap())
-                                .unwrap()
-                                .as_millis()
-                        } else {
-                            0
-                        };
-                    if progress_ms > (track.duration_ms as u128) {
-                        Some(track.duration_ms)
+            Some(ref playback) => {
+                let progress_ms = (playback.progress_ms.unwrap() as u128)
+                    + if playback.is_playing {
+                        std::time::Instant::now()
+                            .saturating_duration_since(self.playback_last_updated.unwrap())
+                            .as_millis()
                     } else {
-                        Some(progress_ms as u32)
-                    }
-                }
-                _ => None,
-            },
+                        0
+                    };
+                Some(progress_ms as u32)
+            }
         }
     }
 
@@ -113,7 +109,7 @@ impl PlayerState {
 impl Default for PlayerState {
     fn default() -> Self {
         Self {
-            auth_token_expires_at: std::time::SystemTime::now(),
+            token: Token::new(),
             devices: vec![],
             user_playlists: vec![],
             user_saved_albums: vec![],
@@ -161,6 +157,16 @@ impl Context {
         }
     }
 
+    /// gets all tracks inside the current playing context (immutable)
+    pub fn get_tracks(&self) -> Option<&Vec<Track>> {
+        match self {
+            Context::Unknown(_) => None,
+            Context::Album(_, ref tracks) => Some(tracks),
+            Context::Playlist(_, ref tracks) => Some(tracks),
+            Context::Artist(_, ref tracks, _, _) => Some(tracks),
+        }
+    }
+
     /// gets all tracks inside the current playing context (mutable)
     pub fn get_tracks_mut(&mut self) -> Option<&mut Vec<Track>> {
         match self {
